package mi.hdm.recipes;

import mi.hdm.exceptions.InvalidCategoryException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

//TODO: make this not a singleton?
public class CategoryManager {
    private static final CategoryManager instance = new CategoryManager();
    private static final Logger log = LogManager.getLogger(CategoryManager.class);

    private final List<Category> allCategories;

    private CategoryManager() {
        allCategories = new ArrayList<>();
    }

    public static CategoryManager getInstance() {
        return instance;
    }

    /**
     * Adds a category if neither the given name nor the color code have been used in a category before.
     * @param name Name for the category that is to be added
     * @param colourCode Color code for the category that is to be added
     * @throws InvalidCategoryException if the category already exists
     */
    public void addCategory(String name, int colourCode){
        Category c = new Category(name, colourCode);
        if (getCategoryByName(name).isPresent()) {
            log.error("Category {} not added because it already exists", c.getName());
            throw new InvalidCategoryException("Category already exists.");
        } else {
            allCategories.add(c);
            log.info("Category {} added successfully.", c.getName());
        }
    }

    public void deleteCategory(Category c) {
        log.info("Category {} deleted successfully.", c.getName());
        if (allCategories.remove(c) == false) {
            throw new InvalidCategoryException("Category is not listed.");
        }
    }

    public void deleteCategory(String name) {
        Category c = getCategoryByName(name).orElseThrow(() -> new InvalidCategoryException("Category not found."));
        deleteCategory(c);
    }

    public List<Category> getAllCategories() {
        return allCategories;
    }

    public void clearCategories() {
        allCategories.clear();
        log.info("List allCategories cleared successfully.");
    }

    private Optional<Category> getCategoryByName(String name) {
        return allCategories.stream()
                .filter(c -> c.getName().equals(name))
                .findFirst();
    }
}
